using System;
using Enyim.Caching;
using Enyim.Caching.Memcached;
using Microsoft.Extensions.Options;
using StackExchange.Redis;

namespace Infrastructure.Cache
{
    /// <summary>
    /// 缓存redis实现
    /// </summary>
    public sealed class RedisCacheContext : ICacheContext
    {
        private ConnectionMultiplexer _conn { get; set; }
        private IDatabase iDatabase { get; set; }

        public RedisCacheContext(IOptions<AppSetting> options)
        {
            _conn = ConnectionMultiplexer.Connect(options.Value.RedisConf.Conn);
            iDatabase = _conn.GetDatabase(options.Value.RedisConf.Database);
        }

        public override T Get<T>(string key)
        {
            RedisValue value = iDatabase.StringGet(key);
            if (!value.HasValue)
            {
                return default(T);
            }

            if (typeof(T) == typeof(string))
            {
                return (T)Convert.ChangeType(value, typeof(T));
            }
            else
            {
                return JsonHelper.Instance.Deserialize<T>(value);
            }
        }

        public override bool Set<T>(string key, T t, DateTime expire)
        {
            if (typeof(T) == typeof(string))
            {
                return iDatabase.StringSet(key, t.ToString(), expire - DateTime.Now);
            }
            else
            {
                return iDatabase.StringSet(key, JsonHelper.Instance.Serialize(t), expire - DateTime.Now);
            }
        }

        public override bool Remove(string key)
        {
            return iDatabase.KeyDelete(key);
        }
        /// <summary>
        /// HSET - 异步设置哈希表中的多个字段。
        /// </summary>
        /// <param name="key"></param>
        /// <param name="hashFields"></param>
        /// <param name="flags"></param>
        public void HashSetAsync(RedisKey key, HashEntry[] hashFields, CommandFlags flags = CommandFlags.None)
        {
            iDatabase.HashSetAsync(key, hashFields, CommandFlags.None);
        }
        /// <summary>
        /// HGETALL - 异步获取哈希表中的所有字段和值。
        /// </summary>
        /// <param name="key"></param>
        /// <param name="flags"></param>
        /// <returns></returns>
        public Task<HashEntry[]> HashGetAllAsync(RedisKey key, CommandFlags flags = CommandFlags.None)
        {
            return iDatabase.HashGetAllAsync(key, CommandFlags.None);
        }
        public override Task<T> Lock<T>(string id, Func<string, Task<T>> func, string key = "hc")
        {
            return func(id);
        }
        public Task<RedisValue[]> SetMembersAsync(RedisKey key, CommandFlags flags = CommandFlags.None)
        {
            return iDatabase.SetMembersAsync(key, CommandFlags.None);
        }
        public Task<bool> SetAddAsync(RedisKey key, RedisValue value, CommandFlags flags = CommandFlags.None)
        {
            return iDatabase.SetAddAsync(key, value, CommandFlags.None);
        }
        public Task<bool> SetRemoveAsync(RedisKey key, RedisValue value, CommandFlags flags = CommandFlags.None)
        {
            return iDatabase.SetRemoveAsync(key, value, CommandFlags.None);
        }
    }
}